@@ -14,11 +14,11 @@
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details. 
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "lime.h"
@@ -50,6 +50,7 @@ char * path = 0;
 int dio = 0;
 int port = 0;
 int localhostonly = 0;
+long timeout = 500;
 
 extern struct resource iomem_resource;
 
@@ -57,6 +58,13 @@ module_param(path, charp, S_IRUGO);
 module_param(dio, int, S_IRUGO);
 module_param(format, charp, S_IRUGO);
 module_param(localhostonly, int, S_IRUGO);
+module_param(timeout, long, S_IRUGO);
+
+#define RETRY_IF_INTURRUPTED(f) ({ \
+	ssize_t err; \
+	do { err = f; } while(err == -EAGAIN || err == -EINTR); \
+	err; \
+})
 
 int init_module (void)
 {
@@ -69,7 +77,7 @@ int init_module (void)
 		DBG("No format parameter specified");
 		return -EINVAL;
 	}
-	
+
 	DBG("Parameters");
 	DBG("  PATH: %s", path);
 	DBG("  DIO: %u", dio);
@@ -99,7 +107,7 @@ static int init() {
 	__PTRDIFF_TYPE__ p_last = -1;
 #endif
 
-	DBG("Initilizing Dump...");
+	DBG("Initializing Dump...");
 
 	if((err = setup())) {
 		DBG("Setup Error");
@@ -123,7 +131,8 @@ static int init() {
 
 		p_last = p->end;
 	}
-	
+
+	DBG("Memory Dump Complete...");
 
 	cleanup();
 
@@ -140,13 +149,13 @@ static int write_lime_header(struct resource * res) {
 	header.version = 1;
 	header.s_addr = res->start;
 	header.e_addr = res->end;
-	
+
 	s = write_vaddr(&header, sizeof(lime_mem_range_header));
-	
+
 	if (s != sizeof(lime_mem_range_header)) {
 		DBG("Error sending header %zd", s);
 		return (int) s;
-	}				
+	}
 
 	return 0;
 }
@@ -177,17 +186,21 @@ static void write_range(struct resource * res) {
 #endif
 	struct page * p;
 	void * v;
-	
+
 	ssize_t s;
+	ktime_t start,end;
+
+	DBG("Writing range %llx - %llx.", res->start, res->end);
 
 	for (i = res->start; i <= res->end; i += is) {
+		start = ktime_get();
 
 		p = pfn_to_page((i) >> PAGE_SHIFT);
-        
+
         is = min((size_t) PAGE_SIZE, (size_t) (res->end - i + 1));
 
         if (is < PAGE_SIZE) {
-        	// We can't map partial pages and 
+        	// We can't map partial pages and
         	// the linux kernel doesn't use them anyway
         	DBG("Padding partial page: vaddr %p size: %lu", (void *) i, (unsigned long) is);
         	write_padding(is);
@@ -204,11 +217,21 @@ static void write_range(struct resource * res) {
 				write_padding(is - s);
 			}
 		}
+
+		end = ktime_get();
+
+		if (timeout > 0 && ktime_to_ms(ktime_sub(end, start)) > timeout) {
+			DBG("Reading is too slow.  Skipping Range...");
+			write_padding(res->end - i - is);
+			break;
+		}
 	}
 }
 
 static ssize_t write_vaddr(void * v, size_t is) {
-	return (method == LIME_METHOD_TCP) ? write_vaddr_tcp(v, is) : write_vaddr_disk(v, is);
+	return RETRY_IF_INTURRUPTED(
+		(method == LIME_METHOD_TCP) ? write_vaddr_tcp(v, is) : write_vaddr_disk(v, is)
+	);
 }
 
 static int setup(void) {
@@ -221,7 +244,7 @@ static void cleanup(void) {
 
 void cleanup_module(void)
 {
-	
+
 }
 
 MODULE_LICENSE("GPL");
